use std::{collections::HashMap, str::FromStr};

use serde::{Deserialize, Serialize};

pub mod bin;
pub mod build;
pub mod fs;
pub mod package;
pub mod pkgr;

#[derive(Serialize, Deserialize)]
#[serde(default)]
pub struct Manifest<P = Option<pkgr::PKGR>> {
    pub package: package::Package,
    pub dependencies: HashMap<String, String>,
    pub fs: fs::FS,
    pub bin: Option<bin::Bin>,
    pub build: Option<build::Build>,
    pub pkgr: P,
}

impl<T> Manifest<T> {
    pub fn valid(&self) -> bool {
        if self.bin.is_none() && self.build.is_none() {
            return false;
        }
        true
    }
}

impl FromStr for Manifest {
    type Err = toml::de::Error;
    fn from_str(s: &str) -> Result<Self, Self::Err> {
        toml::from_str(s)
    }
}

impl Default for Manifest {
    fn default() -> Self {
        Manifest {
            package: package::Package::default(),
            dependencies: HashMap::default(),
            fs: fs::FS::default(),
            bin: None,
            build: None,
            pkgr: None,
        }
    }
}
